import { AutoMap } from '@automapper/classes';
import {
  IsOptional,
  IsString,
  IsBoolean,
  IsDate,
  IsInt,
  IsEnum,
  ValidateNested,
  ValidateIf,
  ArrayMinSize,
  IsNotEmptyObject,
} from 'class-validator';
import { Frequency } from '../types/frequency.enum';
import { Transform, Type } from 'class-transformer';
import { mappingKeyToFrequency } from './utils/frequency.mapping';
import { MarginDTO } from './create.margin.dto';
import { ScheduleDTO } from './create.schedule.dto';
import { AddressDTO } from './create.address.dto';
import { HasProperPassengerSeats } from './has-passenger-seats.validator';
import { HasProperDriverSeats } from './has-driver-seats.validator';

export class CreateAdRequest {
  @IsString()
  @AutoMap()
  userUuid: string;

  @IsOptional()
  @HasProperDriverSeats()
  @IsBoolean()
  @AutoMap()
  driver?: boolean;

  @IsOptional()
  @HasProperPassengerSeats()
  @IsBoolean()
  @AutoMap()
  passenger?: boolean;

  @Transform(({ value }) => mappingKeyToFrequency(value), {
    toClassOnly: true,
  })
  @IsEnum(Frequency)
  @AutoMap()
  frequency: Frequency;

  @ValidateIf((ad) => ad.frequency === 'PUNCTUAL')
  @Type(() => Date)
  @IsDate()
  @AutoMap()
  departure?: Date;

  @ValidateIf((ad) => ad.frequency === 'RECURRENT')
  @Type(() => Date)
  @IsDate()
  @AutoMap()
  fromDate?: Date;

  @ValidateIf((ad) => ad.frequency === 'RECURRENT')
  @Type(() => Date)
  @IsDate()
  @AutoMap()
  toDate?: Date;

  @ValidateIf((ad) => ad.frequency === 'RECURRENT')
  @Type(() => ScheduleDTO)
  @ValidateNested({ each: true })
  @IsNotEmptyObject()
  @AutoMap()
  schedule?: ScheduleDTO;

  @IsOptional()
  @Type(() => MarginDTO)
  @ValidateNested({ each: true })
  @AutoMap()
  marginDurations?: MarginDTO;

  @ValidateIf((ad) => ad.driver)
  @IsInt()
  @AutoMap()
  seatsDriver?: number;

  @IsOptional()
  @IsInt()
  @AutoMap()
  seatsPassenger?: number;

  @IsOptional()
  @IsBoolean()
  @AutoMap()
  strict?: boolean;

  @Type(() => AddressDTO)
  @ArrayMinSize(2)
  @ValidateNested({ each: true })
  @AutoMap(() => [AddressDTO])
  addresses: AddressDTO[];
}
